@@ -83,6 +83,7 @@ from picard.util import (
     find_best_match,
     format_time,
     mbid_validate,
+    process_events_iter,
 )
 from picard.util.imagelist import (
     add_metadata_images,
@@ -170,6 +171,13 @@ class Album(DataObject, Item):
                     file.metadata['musicbrainz_discid'] = track_discids
                     file.update()
 
+    def get_next_track(self, track):
+        try:
+            index = self.tracks.index(track)
+            return self.tracks[index + 1]
+        except (IndexError, ValueError):
+            return None
+
     def get_album_artists(self):
         """Returns the list of album artists (as AlbumArtist objects)"""
         return self._album_artists
@@ -311,7 +319,6 @@ class Album(DataObject, Item):
                 mm = Metadata()
                 mm.copy(self._new_metadata)
                 medium_to_metadata(medium_node, mm)
-                discpregap = False
                 format = medium_node.get('format')
                 if format:
                     all_media.append(format)
@@ -319,29 +326,43 @@ class Album(DataObject, Item):
                 for dj in djmix_ars.get(mm["discnumber"], []):
                     mm.add("djmixer", dj)
 
+                if va:
+                    mm["compilation"] = "1"
+                else:
+                    del mm["compilation"]
+
                 if 'discs' in medium_node:
                     discids = [disc.get('id') for disc in medium_node['discs']]
                     mm['~musicbrainz_discids'] = discids
                     mm['musicbrainz_discid'] = list(self._discids.intersection(discids))
 
                 if "pregap" in medium_node:
-                    discpregap = True
                     absolutetracknumber += 1
-                    track = self._finalize_loading_track(medium_node['pregap'], mm, artists, va, absolutetracknumber, discpregap)
-                    track.metadata['~pregap'] = "1"
+                    mm['~discpregap'] = '1'
+                    extra_metadata = {
+                        '~pregap': '1',
+                        '~absolutetracknumber': absolutetracknumber,
+                    }
+                    self._finalize_loading_track(medium_node['pregap'], mm, artists, extra_metadata)
 
                 track_count = medium_node['track-count']
                 if track_count:
                     tracklist_node = medium_node['tracks']
                     for track_node in tracklist_node:
                         absolutetracknumber += 1
-                        track = self._finalize_loading_track(track_node, mm, artists, va, absolutetracknumber, discpregap)
+                        extra_metadata = {
+                            '~absolutetracknumber': absolutetracknumber,
+                        }
+                        self._finalize_loading_track(track_node, mm, artists, extra_metadata)
 
                 if "data-tracks" in medium_node:
                     for track_node in medium_node['data-tracks']:
                         absolutetracknumber += 1
-                        track = self._finalize_loading_track(track_node, mm, artists, va, absolutetracknumber, discpregap)
-                        track.metadata['~datatrack'] = "1"
+                        extra_metadata = {
+                            '~datatrack': '1',
+                            '~absolutetracknumber': absolutetracknumber,
+                        }
+                        self._finalize_loading_track(track_node, mm, artists, extra_metadata)
 
             totalalbumtracks = absolutetracknumber
             self._new_metadata['~totalalbumtracks'] = totalalbumtracks
@@ -350,8 +371,10 @@ class Album(DataObject, Item):
 
             for track in self._new_tracks:
                 track.metadata["~totalalbumtracks"] = totalalbumtracks
+                track.orig_metadata["~totalalbumtracks"] = totalalbumtracks
                 if len(artists) > 1:
                     track.metadata["~multiartist"] = "1"
+                    track.orig_metadata["~multiartist"] = "1"
             del self._release_node
             del self._release_artist_nodes
             self._tracks_loaded = True
@@ -368,6 +391,7 @@ class Album(DataObject, Item):
                     except ScriptError:
                         log.exception("Failed to run tagger script %s on track", s_name)
                     track.metadata.strip_whitespace()
+                    track.scripted_metadata.update(track.metadata)
                 # Run tagger script for the album itself
                 try:
                     parser.eval(s_text, self._new_metadata)
@@ -404,7 +428,7 @@ class Album(DataObject, Item):
             if self.item.isSelected():
                 self.tagger.window.refresh_metadatabox()
 
-    def _finalize_loading_track(self, track_node, metadata, artists, va, absolutetracknumber, discpregap):
+    def _finalize_loading_track(self, track_node, metadata, artists, extra_metadata=None):
         # As noted in `_parse_release` above, the release artist nodes
         # may contain supplementary data that isn't present in track
         # artist nodes. Similarly, the track artists may contain
@@ -421,18 +445,12 @@ class Album(DataObject, Item):
         tm = track.metadata
         tm.copy(metadata)
         track_to_metadata(track_node, track)
-        tm["~absolutetracknumber"] = absolutetracknumber
-        track.orig_metadata.copy(tm)
         track._customize_metadata()
 
         self._new_metadata.length += tm.length
         artists.add(tm["artist"])
-        if va:
-            tm["compilation"] = "1"
-        else:
-            del tm["compilation"]
-        if discpregap:
-            tm["~discpregap"] = "1"
+        if extra_metadata:
+            tm.update(extra_metadata)
 
         # Run track metadata plugins
         try:
@@ -440,6 +458,7 @@ class Album(DataObject, Item):
         except BaseException:
             self.error_append(traceback.format_exc())
 
+        track.orig_metadata.copy(tm)
         return track
 
     def load(self, priority=False, refresh=False):
@@ -543,12 +562,11 @@ class Album(DataObject, Item):
 
             # try to match by similarity
             def candidates():
-                for track in self.tracks:
+                for track in process_events_iter(self.tracks):
                     yield SimMatchAlbum(
                         similarity=track.metadata.compare(file.orig_metadata),
                         track=track
                     )
-                    QtCore.QCoreApplication.processEvents()
 
             no_match = SimMatchAlbum(similarity=-1, track=self.unmatched_files)
             best_match = find_best_match(candidates, no_match)
@@ -585,7 +603,7 @@ class Album(DataObject, Item):
         return True
 
     def can_view_info(self):
-        return (self.loaded and (self.metadata.images or self.orig_metadata.images)) or self.errors
+        return self.loaded or self.errors
 
     def is_album_like(self):
         return True
