@@ -6,7 +6,7 @@
 # Copyright (C) 2006-2008, 2011 Lukáš Lalinský
 # Copyright (C) 2008 Hendrik van Antwerpen
 # Copyright (C) 2008 Will
-# Copyright (C) 2010-2011, 2014, 2018-2019 Philipp Wolfer
+# Copyright (C) 2010-2011, 2014, 2018-2020 Philipp Wolfer
 # Copyright (C) 2011-2013 Michael Wiencek
 # Copyright (C) 2012 Chad Wilson
 # Copyright (C) 2012 Wieland Hoffmann
@@ -41,6 +41,7 @@ from heapq import (
     heappush,
 )
 import ntpath
+from operator import attrgetter
 import re
 
 from PyQt5 import QtCore
@@ -57,6 +58,7 @@ from picard.util import (
     album_artist_from_path,
     find_best_match,
     format_time,
+    process_events_iter,
 )
 from picard.util.imagelist import (
     add_metadata_images,
@@ -115,18 +117,22 @@ class Cluster(QtCore.QObject, Item):
             self.related_album.update()
 
     def add_files(self, files):
-        for file in files:
+        added_files = set(files) - set(self.files)
+        if not added_files:
+            return
+        for file in added_files:
             self.metadata.length += file.metadata.length
             file._move(self)
             file.update(signal=False)
             if self.can_show_coverart:
                 file.metadata_images_changed.connect(self.update_metadata_images)
-        self.files.extend(files)
+        added_files = sorted(added_files, key=attrgetter('discnumber', 'tracknumber', 'base_filename'))
+        self.files.extend(added_files)
         self.metadata['totaltracks'] = len(self.files)
-        self.item.add_files(files)
+        self.item.add_files(added_files)
         if self.can_show_coverart:
-            add_metadata_images(self, files)
-        self._update_related_album(added_files=files)
+            add_metadata_images(self, added_files)
+        self._update_related_album(added_files=added_files)
 
     def add_file(self, file):
         self.add_files([file])
@@ -137,13 +143,13 @@ class Cluster(QtCore.QObject, Item):
         self.files.remove(file)
         self.metadata['totaltracks'] = len(self.files)
         self.item.remove_file(file)
-        if not self.special and self.get_num_files() == 0:
-            self.tagger.remove_cluster(self)
         if self.can_show_coverart:
             file.metadata_images_changed.disconnect(self.update_metadata_images)
             remove_metadata_images(self, [file])
         self._update_related_album(removed_files=[file])
         self.tagger.window.set_processing(False)
+        if not self.special and self.get_num_files() == 0:
+            self.tagger.remove_cluster(self)
 
     def update(self):
         if self.item:
@@ -278,7 +284,7 @@ class Cluster(QtCore.QObject, Item):
         # 10 evenly spaced indexes of files being clustered, used as checkpoints for every 10% progress
         status_update_steps = ProgressCheckpoints(num_files, 10)
 
-        for i, file in enumerate(files):
+        for i, file in process_events_iter(enumerate(files)):
             artist = file.metadata["albumartist"] or file.metadata["artist"]
             album = file.metadata["album"]
             # Improve clustering from directory structure if no existing tags
@@ -299,7 +305,6 @@ class Cluster(QtCore.QObject, Item):
                     'update': status_update_steps.progress(i),
                 }
                 tagger.window.set_statusbar_message(statusmsg, mparams)
-                QtCore.QCoreApplication.processEvents()
 
         artist_cluster_engine = ClusterEngine(artist_dict, ClusterType.ARTIST)
         artist_cluster_engine.cluster(threshold, tagger)
@@ -530,7 +535,7 @@ class ClusterEngine(object):
         # 20 evenly spaced indexes of files being clustered, used as checkpoints for every 5% progress
         status_update_steps = ProgressCheckpoints(num_files, 20)
 
-        for y in range(num_files):
+        for y in process_events_iter(range(num_files)):
             token_y = self.cluster_dict.get_token(y).lower()
             for x in range(y):
                 if x != y:
@@ -553,7 +558,6 @@ class ClusterEngine(object):
                     'update': status_update_steps.progress(y),
                 }
                 tagger.window.set_statusbar_message(statusmsg, mparams)
-                QtCore.QCoreApplication.processEvents()
 
         for i in range(len(heap)):
             c, pair = heappop(heap)
