@@ -47,7 +47,6 @@ from collections import defaultdict
 from functools import partial
 from itertools import chain
 import logging
-from operator import attrgetter
 import os.path
 import platform
 import re
@@ -113,6 +112,7 @@ from picard.util import (
     encode_filename,
     is_hidden,
     mbid_validate,
+    process_events_iter,
     thread,
     uniqify,
     versions,
@@ -277,7 +277,7 @@ class Tagger(QtWidgets.QApplication):
         self.mbid_redirects = {}
         self.unclustered_files = UnclusteredFiles()
         self.nats = None
-        self.window = MainWindow()
+        self.window = MainWindow(disable_player=picard_args.no_player)
         self.exit_cleanup = []
         self.stopping = False
 
@@ -385,14 +385,8 @@ class Tagger(QtWidgets.QApplication):
 
     def _run_init(self):
         if self._cmdline_files:
-            files = []
-            for file in self._cmdline_files:
-                if os.path.isdir(file):
-                    self.add_directory(decode_filename(file))
-                else:
-                    files.append(decode_filename(file))
-            if files:
-                self.add_files(files)
+            files = [decode_filename(f) for f in self._cmdline_files]
+            self.add_paths(files)
             del self._cmdline_files
 
     def run(self):
@@ -409,10 +403,7 @@ class Tagger(QtWidgets.QApplication):
             event.run()
         elif event.type() == QtCore.QEvent.FileOpen:
             file = event.file()
-            if os.path.isdir(file):
-                self.add_directory(file)
-            else:
-                self.add_files([file])
+            self.add_paths([file])
             if IS_HAIKU:
                 self.bring_tagger_front()
             # We should just return True here, except that seems to
@@ -463,7 +454,7 @@ class Tagger(QtWidgets.QApplication):
                 return
 
         if target:
-            self.move_files([file], target)
+            self.move_file(file, target)
         else:
             self.unclustered_files.add_file(file)
 
@@ -472,29 +463,48 @@ class Tagger(QtWidgets.QApplication):
             log.debug("Trying to analyze %r ...", file)
             self.analyze([file])
 
-    def move_files(self, files, target):
+    def move_file(self, file, target):
+        if target is None:
+            log.debug("Aborting move since target is invalid")
+            return
+        if isinstance(target, Album):
+            self.move_files_to_album([file], album=target)
+        elif isinstance(target, ClusterList):
+            if isinstance(file.parent, Track):
+                file.parent.remove_file(file)
+            self.cluster([file])
+        # To be able to move a file to it target must implement add_file(file)
+        elif hasattr(target, 'add_file'):
+            file.move(target)
+
+    def move_files(self, files, target, move_to_multi_tracks=True):
         if target is None:
             log.debug("Aborting move since target is invalid")
             return
         self.window.set_sorting(False)
-        if isinstance(target, (Track, Cluster)):
-            for file in files:
+        if isinstance(target, Cluster):
+            for file in process_events_iter(files):
                 file.move(target)
-                QtCore.QCoreApplication.processEvents()
+        elif isinstance(target, Track):
+            album = target.album
+            for file in process_events_iter(files):
+                file.move(target)
+                if move_to_multi_tracks:  # Assign next file to following track
+                    target = album.get_next_track(target) or album.unmatched_files
         elif isinstance(target, File):
-            for file in files:
+            for file in process_events_iter(files):
                 file.move(target.parent)
-                QtCore.QCoreApplication.processEvents()
         elif isinstance(target, Album):
             self.move_files_to_album(files, album=target)
         elif isinstance(target, ClusterList):
+            for file in process_events_iter(files):
+                if isinstance(file.parent, Track):
+                    file.parent.remove_file(file)
             self.cluster(files)
         self.window.set_sorting(True)
 
-    def add_files(self, filenames, target=None, result=None):
+    def add_files(self, filenames, target=None):
         """Add files to the tagger."""
-        if result:
-            filenames = result   # Handles add_directory task results coming from a worker thread
         ignoreregex = None
         pattern = config.setting['ignore_regex']
         if pattern:
@@ -527,54 +537,40 @@ class Tagger(QtWidgets.QApplication):
             new_files.sort(key=lambda x: x.filename)
             self.window.set_sorting(False)
             self._pending_files_count += len(new_files)
-            for file in new_files:
+            for i, file in enumerate(new_files):
                 file.load(partial(self._file_loaded, target=target))
-                QtCore.QCoreApplication.processEvents()
-
-    def _scan_dir(self, folders, recursive, ignore_hidden):
-        files = []
-        local_folders = list(folders)
-        while local_folders:
-            current_folder = local_folders.pop(0)
-            current_folder_files = []
+                # Calling processEvents helps processing the _file_loaded
+                # callbacks in between, which keeps the UI more responsive.
+                # Avoid calling it to often to not slow down the loading to much
+                # Using an uneven number to have the unclustered file counter
+                # not look stuck in certain digits.
+                if i % 17 == 0:
+                    QtCore.QCoreApplication.processEvents()
 
+    @staticmethod
+    def _scan_paths_recursive(paths, recursive, ignore_hidden):
+        local_paths = list(paths)
+        while local_paths:
+            current_path = local_paths.pop(0)
             try:
-                for entry in os.scandir(current_folder):
-                    if ignore_hidden and is_hidden(entry.path):
-                        continue
-                    if recursive and entry.is_dir():
-                        local_folders.append(entry.path)
-                    else:
-                        current_folder_files.append(entry.path)
-            except FileNotFoundError:
-                pass
+                if os.path.isdir(current_path):
+                    for entry in os.scandir(current_path):
+                        if ignore_hidden and is_hidden(entry.path):
+                            continue
+                        if recursive and entry.is_dir():
+                            local_paths.append(entry.path)
+                        else:
+                            yield entry.path
+                else:
+                    yield current_path
+            except OSError as err:
+                log.warning(err)
 
-            number_of_files = len(current_folder_files)
-            if number_of_files:
-                mparms = {
-                    'count': number_of_files,
-                    'directory': current_folder,
-                }
-                log.debug("Adding %(count)d files from '%(directory)r'" %
-                          mparms)
-                self.window.set_statusbar_message(
-                    ngettext(
-                        "Adding %(count)d file from '%(directory)s' ...",
-                        "Adding %(count)d files from '%(directory)s' ...",
-                        number_of_files),
-                    mparms,
-                    translate=None,
-                    echo=None
-                )
-                files.extend(current_folder_files)
-                QtCore.QCoreApplication.processEvents()
-        return files
-
-    def add_directory(self, path):
-        files = self._scan_dir([path],
+    def add_paths(self, paths, target=None):
+        files = self._scan_paths_recursive(paths,
                             config.setting['recursively_add_files'],
                             config.setting["ignore_hidden_files"])
-        self.add_files(files)
+        self.add_files(files, target=target)
 
     def get_file_lookup(self):
         """Return a FileLookup object."""
@@ -844,10 +840,9 @@ class Tagger(QtWidgets.QApplication):
         cluster_files = defaultdict(list)
         for name, artist, files in Cluster.cluster(files, 1.0, self):
             cluster = self.load_cluster(name, artist)
-            cluster_files[cluster].extend(sorted(files, key=attrgetter('discnumber', 'tracknumber', 'base_filename')))
-        for cluster, files in cluster_files.items():
+            cluster_files[cluster].extend(files)
+        for cluster, files in process_events_iter(cluster_files.items()):
             cluster.add_files(files)
-            QtCore.QCoreApplication.processEvents()
         self.window.set_sorting(True)
 
     def load_cluster(self, name, artist):
@@ -925,6 +920,8 @@ def process_picard_args():
                         help="location of the configuration file")
     parser.add_argument("-d", "--debug", action='store_true',
                         help="enable debug-level logging")
+    parser.add_argument("-M", "--no-player", action='store_true',
+                        help="disable built-in media player")
     parser.add_argument("-N", "--no-restore", action='store_true',
                         help="do not restore positions and/or sizes")
     parser.add_argument("-P", "--no-plugins", action='store_true',
