@@ -45,7 +45,6 @@ from heapq import (
     heappush,
 )
 import os
-import re
 
 from PyQt5 import (
     QtCore,
@@ -73,7 +72,6 @@ from picard.track import (
     Track,
 )
 from picard.util import (
-    encode_filename,
     icontheme,
     natsort,
     restore_method,
@@ -186,6 +184,7 @@ class MainPanel(QtWidgets.QSplitter):
 
     def __init__(self, window, parent=None):
         super().__init__(parent)
+        self.setChildrenCollapsible(False)
         self.window = window
         self.create_icons()
         self._views = [FileTreeView(window, self), AlbumTreeView(window, self)]
@@ -324,31 +323,12 @@ def paint_fingerprint_icon(painter, rect, icon):
     if not icon:
         return
     size = COLUMN_ICON_SIZE
-    padding_h = (rect.width() - size) / 2
+    padding_h = COLUMN_ICON_BORDER
     padding_v = (rect.height() - size) / 2
     target_rect = QtCore.QRect(rect.x() + padding_h, rect.y() + padding_v, size, size)
     painter.drawPixmap(target_rect, icon.pixmap(size, size))
 
 
-class FingerprintColumnWidget(QtWidgets.QWidget):
-    def __init__(self, file, parent=None):
-        super().__init__(parent=parent)
-        self._file = file
-
-    def paintEvent(self, event=None):
-        painter = QtGui.QPainter(self)
-        paint_fingerprint_icon(painter, self.rect(), self.decide_icon())
-
-    def decide_icon(self):
-        if getattr(self._file, 'acoustid_fingerprint', None):
-            if self.tagger.acoustidmanager.is_submitted(self._file):
-                return FileItem.icon_fingerprint_gray
-            else:
-                return FileItem.icon_fingerprint
-        else:
-            return None
-
-
 class ConfigurableColumnsHeader(TristateSortHeaderView):
 
     def __init__(self, parent=None):
@@ -379,7 +359,7 @@ class ConfigurableColumnsHeader(TristateSortHeaderView):
             self._visible_columns.add(column)
             if column == MainPanel.FINGERPRINT_COLUMN:
                 self.setSectionResizeMode(column, QtWidgets.QHeaderView.Fixed)
-                self.parent().resizeColumnToContents(column)
+                self.resizeSection(column, COLUMN_ICON_SIZE)
             else:
                 self.setSectionResizeMode(column, QtWidgets.QHeaderView.Interactive)
         elif column in self._visible_columns:
@@ -395,21 +375,30 @@ class ConfigurableColumnsHeader(TristateSortHeaderView):
 
     def contextMenuEvent(self, event):
         menu = QtWidgets.QMenu(self)
+        parent = self.parent()
 
         for i, column in enumerate(MainPanel.columns):
             if i == 0:
                 continue
-            action = QtWidgets.QAction(_(column[0]), self.parent())
+            action = QtWidgets.QAction(_(column[0]), parent)
             action.setCheckable(True)
             action.setChecked(i in self._visible_columns)
+            action.setEnabled(not self.is_locked)
             action.triggered.connect(partial(self.show_column, i))
             menu.addAction(action)
 
         menu.addSeparator()
-        restore_action = QtWidgets.QAction(_('Restore default columns'), self.parent())
+        restore_action = QtWidgets.QAction(_('Restore default columns'), parent)
+        restore_action.setEnabled(not self.is_locked)
         restore_action.triggered.connect(self.restore_defaults)
         menu.addAction(restore_action)
 
+        lock_action = QtWidgets.QAction(_('Lock columns'), parent)
+        lock_action.setCheckable(True)
+        lock_action.setChecked(self.is_locked)
+        lock_action.toggled.connect(self.lock)
+        menu.addAction(lock_action)
+
         menu.exec_(event.globalPos())
         event.accept()
 
@@ -429,6 +418,12 @@ class ConfigurableColumnsHeader(TristateSortHeaderView):
         if index == MainPanel.FINGERPRINT_COLUMN:
             self.setSortIndicator(-1, QtCore.Qt.AscendingOrder)
 
+    def lock(self, is_locked):
+        super().lock(is_locked)
+        fingerprint_column_index = MainPanel.FINGERPRINT_COLUMN
+        if not self.is_locked and self.count() > fingerprint_column_index:
+            self.setSectionResizeMode(fingerprint_column_index, QtWidgets.QHeaderView.Fixed)
+
 
 class BaseTreeView(QtWidgets.QTreeWidget):
 
@@ -437,7 +432,8 @@ class BaseTreeView(QtWidgets.QTreeWidget):
         self.setHeader(ConfigurableColumnsHeader(self))
         self.window = window
         self.panel = parent
-
+        # Should multiple files dropped be assigned to tracks sequentially?
+        self._move_to_multi_tracks = True
         self.setHeaderLabels([_(h) if n != '~fingerprint' else ''
                               for h, n in MainPanel.columns])
         self.restore_state()
@@ -536,7 +532,7 @@ class BaseTreeView(QtWidgets.QTreeWidget):
             loading.setDisabled(True)
             bottom_separator = True
 
-            if len(self.selectedItems()) == 1:
+            if len(self.selectedItems()) == 1 and obj.release_group:
                 def _add_other_versions():
                     releases_menu.removeAction(loading)
                     heading = releases_menu.addAction(obj.release_group.version_headings)
@@ -648,9 +644,11 @@ class BaseTreeView(QtWidgets.QTreeWidget):
     @restore_method
     def restore_state(self):
         self._restore_state(config.persist[self.header_state.name])
+        self.header().lock(config.persist[self.header_locked.name])
 
     def save_state(self):
         config.persist[self.header_state.name] = self.header().saveState()
+        config.persist[self.header_locked.name] = self.header().is_locked
 
     def restore_default_columns(self):
         self._restore_state(None)
@@ -720,36 +718,32 @@ class BaseTreeView(QtWidgets.QTreeWidget):
         hscrollbar.setValue(xpos)
 
     @staticmethod
-    def drop_urls(urls, target):
+    def drop_urls(urls, target, move_to_multi_tracks=True):
         files = []
-        new_files = []
+        new_paths = []
+        tagger = QtCore.QObject.tagger
         for url in urls:
             log.debug("Dropped the URL: %r", url.toString(QtCore.QUrl.RemoveUserInfo))
             if url.scheme() == "file" or not url.scheme():
                 filename = os.path.normpath(os.path.realpath(url.toLocalFile().rstrip("\0")))
-                file = BaseTreeView.tagger.files.get(filename)
+                file = tagger.files.get(filename)
                 if file:
                     files.append(file)
-                elif os.path.isdir(encode_filename(filename)):
-                    BaseTreeView.tagger.add_directory(filename)
                 else:
-                    new_files.append(filename)
+                    new_paths.append(filename)
             elif url.scheme() in ("http", "https"):
-                path = url.path()
-                match = re.search(r"/(release|recording)/([0-9a-z\-]{36})", path)
-                if match:
-                    entity = match.group(1)
-                    mbid = match.group(2)
-                    if entity == "release":
-                        BaseTreeView.tagger.load_album(mbid)
-                    elif entity == "recording":
-                        BaseTreeView.tagger.load_nat(mbid)
+                file_lookup = tagger.get_file_lookup()
+                file_lookup.mbid_lookup(url.path(), browser_fallback=False)
         if files:
-            BaseTreeView.tagger.move_files(files, target)
-        if new_files:
-            BaseTreeView.tagger.add_files(new_files, target=target)
+            tagger.move_files(files, target, move_to_multi_tracks)
+        if new_paths:
+            tagger.add_paths(new_paths, target=target)
 
     def dropEvent(self, event):
+        # Dropping with Alt key pressed forces all dropped files being
+        # assigned to the same track.
+        if event.keyboardModifiers() == QtCore.Qt.AltModifier:
+            self._move_to_multi_tracks = False
         return QtWidgets.QTreeView.dropEvent(self, event)
 
     def dropMimeData(self, parent, index, data, action):
@@ -769,7 +763,7 @@ class BaseTreeView(QtWidgets.QTreeWidget):
         urls = data.urls()
         if urls:
             # Use QTimer.singleShot to run expensive processing outside of the drop handler.
-            QtCore.QTimer.singleShot(0, partial(self.drop_urls, urls, target))
+            QtCore.QTimer.singleShot(0, partial(self.drop_urls, urls, target, self._move_to_multi_tracks))
             handled = True
         # application/picard.album-list
         albums = data.data("application/picard.album-list")
@@ -779,6 +773,7 @@ class BaseTreeView(QtWidgets.QTreeWidget):
             QtCore.QTimer.singleShot(0, partial(self.tagger.move_files,
                 self.tagger.get_files_from_objects(albums), target))
             handled = True
+        self._move_to_multi_tracks = True  # Reset for next drop
         return handled
 
     def activate_item(self, index):
@@ -809,6 +804,7 @@ class BaseTreeView(QtWidgets.QTreeWidget):
 class FileTreeView(BaseTreeView):
 
     header_state = config.Option("persist", "file_view_header_state", QtCore.QByteArray())
+    header_locked = config.BoolOption("persist", "file_view_header_locked", False)
 
     def __init__(self, window, parent=None):
         super().__init__(window, parent)
@@ -839,6 +835,7 @@ class FileTreeView(BaseTreeView):
 class AlbumTreeView(BaseTreeView):
 
     header_state = config.Option("persist", "album_view_header_state", QtCore.QByteArray())
+    header_locked = config.BoolOption("persist", "album_view_header_locked", False)
 
     def __init__(self, window, parent=None):
         super().__init__(window, parent)
@@ -881,7 +878,7 @@ class TreeItem(QtWidgets.QTreeWidgetItem):
             MainPanel.DISCNUMBER_COLUMN,
         ]:
             self.setTextAlignment(column, QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
-        self.setSizeHint(MainPanel.TITLE_COLUMN, ICON_SIZE)
+        self.setSizeHint(MainPanel.FINGERPRINT_COLUMN, ICON_SIZE)
 
     def setText(self, column, text):
         self._sortkeys[column] = None
@@ -1020,7 +1017,6 @@ class TrackItem(TreeItem):
 
     def update(self, update_album=True, update_files=True):
         track = self.obj
-        tree_widget = self.treeWidget()
         if track.num_linked_files == 1:
             file = track.linked_files[0]
             file.item = self
@@ -1030,16 +1026,13 @@ class TrackItem(TreeItem):
             self.setToolTip(MainPanel.TITLE_COLUMN, _(FileItem.decide_file_icon_info(file)))
             self.takeChildren()
             self.setExpanded(False)
-            self.setToolTip(MainPanel.FINGERPRINT_COLUMN, FileItem.decide_fingerprint_icon_info(file))
-            if tree_widget:
-                if not tree_widget.itemWidget(self, MainPanel.FINGERPRINT_COLUMN):
-                    tree_widget.setItemWidget(self, MainPanel.FINGERPRINT_COLUMN,
-                        FingerprintColumnWidget(file=file))
+            fingerprint_icon, fingerprint_tooltip = FileItem.decide_fingerprint_icon_info(file)
+            self.setToolTip(MainPanel.FINGERPRINT_COLUMN, fingerprint_tooltip)
+            self.setIcon(MainPanel.FINGERPRINT_COLUMN, fingerprint_icon)
         else:
             self.setToolTip(MainPanel.TITLE_COLUMN, "")
             self.setToolTip(MainPanel.FINGERPRINT_COLUMN, "")
-            if tree_widget:
-                tree_widget.setItemWidget(self, MainPanel.FINGERPRINT_COLUMN, None)
+            self.setIcon(MainPanel.FINGERPRINT_COLUMN, QtGui.QIcon())
             if track.ignored_for_completeness():
                 color = TreeItem.text_color_secondary
             else:
@@ -1092,18 +1085,15 @@ class FileItem(TreeItem):
     def update(self, update_track=True):
         file = self.obj
         self.setIcon(MainPanel.TITLE_COLUMN, FileItem.decide_file_icon(file))
-        self.setToolTip(MainPanel.FINGERPRINT_COLUMN, self.decide_fingerprint_icon_info(file))
+        fingerprint_icon, fingerprint_tooltip = FileItem.decide_fingerprint_icon_info(file)
+        self.setToolTip(MainPanel.FINGERPRINT_COLUMN, fingerprint_tooltip)
+        self.setIcon(MainPanel.FINGERPRINT_COLUMN, fingerprint_icon)
         color = FileItem.file_colors[file.state]
         bgcolor = get_match_color(file.similarity, TreeItem.base_color)
         for i, column in enumerate(MainPanel.columns):
             self.setText(i, file.column(column[1]))
             self.setForeground(i, color)
             self.setBackground(i, bgcolor)
-        tree_widget = self.treeWidget()
-        if tree_widget:
-            if not tree_widget.itemWidget(self, MainPanel.FINGERPRINT_COLUMN):
-                tree_widget.setItemWidget(self, MainPanel.FINGERPRINT_COLUMN,
-                    FingerprintColumnWidget(file=file))
         if self.isSelected():
             TreeItem.window.update_selection(new_selection=False)
         parent = self.parent()
@@ -1141,10 +1131,14 @@ class FileItem(TreeItem):
 
     @staticmethod
     def decide_fingerprint_icon_info(file):
-        if file.acoustid_fingerprint:
+        if getattr(file, 'acoustid_fingerprint', None):
             if QtCore.QObject.tagger.acoustidmanager.is_submitted(file):
-                return _('Fingerprint has already been submitted')
+                icon = FileItem.icon_fingerprint_gray
+                tooltip = _('Fingerprint has already been submitted')
             else:
-                return _('Unsubmitted fingerprint')
+                icon = FileItem.icon_fingerprint
+                tooltip = _('Unsubmitted fingerprint')
         else:
-            return _('No fingerprint was calculated for this file, use "Scan" or "Generate AcoustID Fingerprints" to calculate the fingerprint.')
+            icon = QtGui.QIcon()
+            tooltip = _('No fingerprint was calculated for this file, use "Scan" or "Generate AcoustID Fingerprints" to calculate the fingerprint.')
+        return (icon, tooltip)
