@@ -1,7 +1,33 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006 Lukáš Lalinský
+#
+# Copyright (C) 2006-2008, 2011-2012, 2014 Lukáš Lalinský
+# Copyright (C) 2007 Nikolai Prokoschenko
+# Copyright (C) 2008 Gary van der Merwe
+# Copyright (C) 2008 Robert Kaye
+# Copyright (C) 2008 Will
+# Copyright (C) 2008-2010, 2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2009 Carlin Mangar
+# Copyright (C) 2009 David Hilton
+# Copyright (C) 2011-2012 Chad Wilson
+# Copyright (C) 2011-2013, 2015-2017 Wieland Hoffmann
+# Copyright (C) 2011-2014 Michael Wiencek
+# Copyright (C) 2013-2014, 2017 Sophist-UK
+# Copyright (C) 2013-2020 Laurent Monin
+# Copyright (C) 2015 Ohm Patel
+# Copyright (C) 2015 samithaj
+# Copyright (C) 2016 Rahul Raturi
+# Copyright (C) 2016 Simon Legner
+# Copyright (C) 2016-2017 Sambhav Kothari
+# Copyright (C) 2017 Antonio Larrosa
+# Copyright (C) 2017 Frederik “Freso” S. Olesen
+# Copyright (C) 2018 Bob Swift
+# Copyright (C) 2018 Kartik Ohri
+# Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2018 virusMac
+# Copyright (C) 2019 Timur Enikeev
+# Copyright (C) 2020 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,6 +43,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from collections import OrderedDict
 import datetime
 from functools import partial
@@ -54,6 +81,7 @@ from picard.util.cdrom import (
 )
 
 from picard.ui import PreserveGeometry
+from picard.ui.aboutdialog import AboutDialog
 from picard.ui.coverartbox import CoverArtBox
 from picard.ui.filebrowser import FileBrowser
 from picard.ui.infodialog import (
@@ -74,7 +102,6 @@ from picard.ui.passworddialog import (
     PasswordDialog,
     ProxyDialog,
 )
-from picard.ui.playertoolbar import Player
 from picard.ui.searchdialog.album import AlbumSearchDialog
 from picard.ui.searchdialog.track import TrackSearchDialog
 from picard.ui.statusindicator import DesktopStatusIndicator
@@ -110,17 +137,19 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config.IntOption("persist", "mediaplayer_volume", 50),
     ]
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, disable_player=False):
         super().__init__(parent)
         self.selected_objects = []
         self.ignore_selection_changes = False
         self.toolbar = None
         self.player = None
         self.status_indicators = []
-        player = Player(self)
-        if player.available:
-            self.player = player
-            self.player.error.connect(self._on_player_error)
+        if not disable_player:
+            from picard.ui.playertoolbar import Player
+            player = Player(self)
+            if player.available:
+                self.player = player
+                self.player.error.connect(self._on_player_error)
         self.setupUi()
 
     def setupUi(self):
@@ -147,8 +176,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             self.search_toolbar.setMovable(False)
 
         mainLayout = QtWidgets.QSplitter(QtCore.Qt.Vertical)
+        mainLayout.setChildrenCollapsible(False)
         mainLayout.setContentsMargins(0, 0, 0, 0)
-        mainLayout.setHandleWidth(1)
 
         self.panel = MainPanel(self, mainLayout)
         self.file_browser = FileBrowser(self.panel)
@@ -183,6 +212,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         for function in ui_init:
             function(self)
 
+    def set_processing(self, processing=True):
+        self.panel.set_processing(processing)
+
+    def set_sorting(self, sorting=True):
+        self.panel.set_sorting(sorting)
+        # self.panel.collapse_clusters(not sorting)
+
     def keyPressEvent(self, event):
         # On macOS Command+Backspace triggers the so called "Forward Delete".
         # It should be treated the same as the Delete button.
@@ -341,7 +377,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             return hasattr(obj, 'keys') and hasattr(obj, '__getitem__')
 
         echo = kwargs.get('echo', log.debug)
-        # _ is defined using builtins.__dict__, so setting it as default named argument
+        # _ is defined using builtins.__dict__, so setting it as default named argument
         # value doesn't work as expected
         translate = kwargs.get('translate', _)
         timeout = kwargs.get('timeout', 0)
@@ -365,9 +401,19 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def _on_submit_acoustid(self):
         if self.tagger.use_acoustid:
             if not config.setting["acoustid_apikey"]:
-                QtWidgets.QMessageBox.warning(self,
-                    _("Submission Error"),
-                    _("You need to configure your AcoustID API key before you can submit fingerprints."))
+                msg = QtWidgets.QMessageBox(self)
+                msg.setIcon(QtWidgets.QMessageBox.Information)
+                msg.setWindowModality(QtCore.Qt.WindowModal)
+                msg.setWindowTitle(_("AcoustID submission not configured"))
+                msg.setText(_(
+                    "You need to configure your AcoustID API key before you can submit fingerprints."))
+                open_options = QtWidgets.QPushButton(
+                    icontheme.lookup('preferences-desktop'), _("Open AcoustID options"))
+                msg.addButton(QtWidgets.QMessageBox.Cancel)
+                msg.addButton(open_options, QtWidgets.QMessageBox.YesRole)
+                msg.exec_()
+                if msg.clickedButton() == open_options:
+                    self.show_options("fingerprinting")
             else:
                 self.tagger.acoustidmanager.submit()
 
@@ -440,7 +486,6 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
         self.remove_action = QtWidgets.QAction(icontheme.lookup('list-remove'), _("&Remove"), self)
         self.remove_action.setStatusTip(_("Remove selected files/albums"))
-        self.remove_action.setShortcut(QtGui.QKeySequence.Delete)
         self.remove_action.setEnabled(False)
         self.remove_action.triggered.connect(self.remove)
 
@@ -509,7 +554,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.generate_fingerprints_action.setStatusTip(_("Generate the AcoustID audio fingerprints for the selected files without doing a lookup"))
         self.generate_fingerprints_action.setEnabled(False)
         self.generate_fingerprints_action.setToolTip(_('Generate the AcoustID audio fingerprints for the selected files'))
-        self.generate_fingerprints_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Alt+F")))
+        self.generate_fingerprints_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+Y")))
         self.generate_fingerprints_action.triggered.connect(self.generate_fingerprints)
 
         self.cluster_action = QtWidgets.QAction(icontheme.lookup('picard-cluster'), _("Cl&uster"), self)
@@ -553,7 +598,10 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.enable_tag_saving_action.setChecked(not config.setting["dont_write_tags"])
         self.enable_tag_saving_action.triggered.connect(self.toggle_tag_saving)
 
-        self.tags_from_filenames_action = QtWidgets.QAction(_("Tags From &File Names..."), self)
+        self.tags_from_filenames_action = QtWidgets.QAction(icontheme.lookup('picard-tags-from-filename'), _("Tags From &File Names..."), self)
+        self.tags_from_filenames_action.setIconText(_("Parse File Names..."))
+        self.tags_from_filenames_action.setToolTip(_('Set tags based on the file names'))
+        self.tags_from_filenames_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+T")))
         self.tags_from_filenames_action.triggered.connect(self.open_tags_from_filenames)
         self.tags_from_filenames_action.setEnabled(False)
 
@@ -828,9 +876,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         tab_order(self.search_edit, self.search_button)
         # Panels
         tab_order(self.search_button, self.file_browser)
-        tab_order(self.file_browser, self.panel.views[0])
-        tab_order(self.panel.views[0], self.panel.views[1])
-        tab_order(self.panel.views[1], self.metadata_box)
+        self.panel.tab_order(tab_order, self.file_browser, self.metadata_box)
 
     def enable_submit(self, enabled):
         """Enable/disable the 'Submit fingerprints' action."""
@@ -908,14 +954,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                     {'directory': dir_list[0]}
                 )
 
-            for directory in dir_list:
-                self.tagger.add_directory(directory)
+            self.tagger.add_paths(dir_list)
 
     def close_active_window(self):
         self.tagger.activeWindow().close()
 
     def show_about(self):
-        return self.show_options("about")
+        return AboutDialog.show_instance(self)
 
     def show_options(self, page=None):
         return OptionsDialog.show_instance(page, self)
@@ -1080,7 +1125,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.tags_from_filenames_action.setEnabled(bool(files))
         self.track_search_action.setEnabled(have_objects)
 
-    def update_selection(self, objects=None):
+    def update_selection(self, objects=None, new_selection=True, drop_album_caches=False):
         if self.ignore_selection_changes:
             return
 
@@ -1144,11 +1189,16 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             elif obj.can_show_coverart:
                 metadata = obj.metadata
 
-        self.metadata_box.selection_dirty = True
-        self.metadata_box.update()
+        if new_selection:
+            self.metadata_box.selection_dirty = True
+        self.metadata_box.update(drop_album_caches=drop_album_caches)
         self.cover_art_box.set_metadata(metadata, orig_metadata, obj)
         self.selection_updated.emit(objects)
 
+    def refresh_metadatabox(self):
+        self.tagger.window.metadata_box.selection_dirty = True
+        self.tagger.window.metadata_box.update()
+
     def show_cover_art(self):
         """Show/hide the cover art box."""
         if self.show_cover_art_action.isChecked():
